@@ -19,8 +19,8 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 #include <string>
 
 #include "command.h"
@@ -32,13 +32,14 @@ cmd_clear clear;
 
 static cmd_options cmd_clear_options[] =
 {
-  NULL,0,0
+  {0,0,0}
 };
 
 
 cmd_clear::cmd_clear(void)
 { 
   name = "clear";
+  abbreviation = "cl";
 
     brief_doc = string("Remove a break point");
 
@@ -52,10 +53,33 @@ display the currently set break points.\n\
 }
 
 
-void cmd_clear::clear(int bp_number)
+void cmd_clear::clear(Expression *expr)
 {
 
-  bp.clear(bp_number);
+  if(expr) {
+
+    Value *v = expr->evaluate();
+    if(v) {
+
+      if (typeid(*v) == typeid(String)) {
+        char szParam[20];
+        ((String*)v)->get(szParam, 20);
+        if(strcmp(szParam, "all") == 0) {
+          bp.clear_all(get_active_cpu());
+        }
+      }
+      else if (typeid(*v) == typeid(Integer)) {
+        /* for now, assume that the expression evaluates to an integer
+	         (later, things like 'clear all' will be add)
+        */
+        gint64 i;
+        v->get(i);
+        bp.clear((unsigned int)i);
+      }
+      delete v;
+    }
+    delete expr;
+  }
 
 }
 
