@@ -25,10 +25,16 @@ Boston, MA 02111-1307, USA.  */
 
 extern int gpsim_read (char *buf, unsigned max_size);
 extern void initialize_readline (void);
+extern void exit_gpsim(void);
+void SetLastFullCommand(const char *pCmd);
+const char * GetLastFullCommand();
+void EnableSTCEcho(bool bEnable);
 
-extern int input_mode;
 #ifndef HAVE_GUI
 extern char *gnu_readline (char *s, unsigned int force_readline);
 #endif
 
+class Processor;
+int gpsim_open(Processor *cpu, const char *file, const char *pProcessorType);
+
 #endif
