@@ -22,7 +22,7 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __16BIT_INSTRUCTIONS_H__
 #define __16BIT_INSTRUCTIONS_H__
 
-#include "pic-instructions.h"
+#include "14bit-instructions.h"
 #include "16bit-registers.h"
 
 /*---------------------------------------------------------
@@ -43,14 +43,15 @@ Boston, MA 02111-1307, USA.  */
 class Branching : public instruction
 {
 public:
-  int destination;
-  unsigned int absolute_destination;
+  int destination_index;
+  unsigned int absolute_destination_index;
 
   virtual void execute(void){ };
   virtual void debug(void){ };
-  virtual char *name(char *);
+  virtual char *name(char *,int);
+  virtual bool isBase() { return true;}
 
-  void decode(pic_processor *new_cpu, unsigned int new_opcode);
+  void decode(Processor *new_cpu, unsigned int new_opcode);
 
 };
 
@@ -59,23 +60,26 @@ class multi_word_instruction : public instruction
 {
  public:
   unsigned int word2_opcode;
-  unsigned int address;
+  unsigned int PMaddress;
+  unsigned int PMindex;
   bool initialized;
 
   virtual int instruction_size(void) { return 2;}
   virtual INSTRUCTION_TYPES isa(void) {return MULTIWORD_INSTRUCTION;};
+  virtual bool isBase() { return true;}
 
+  virtual void initialize(bool init_state) { initialized = init_state; }
 };
 
 //---------------------------------------------------------
 class multi_word_branch : public multi_word_instruction
 {
  public:
-  unsigned int destination;
+  unsigned int destination_index;
 
   void runtime_initialize(void);
   virtual void execute(void){};
-  char *name(char *str);
+  virtual char *name(char *,int);
 
 };
 
@@ -84,11 +88,11 @@ class ADDLW16 : public ADDLW
 {
 
 public:
-  ADDLW16(pic_processor *new_cpu, unsigned int new_opcode) :
+  ADDLW16(Processor *new_cpu, unsigned int new_opcode) :
     ADDLW(new_cpu,  new_opcode)
     {};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new ADDLW16(new_cpu,new_opcode);}
 };
 
@@ -98,9 +102,9 @@ class ADDWF16 : public ADDWF
 public:
   int i;
 
-  ADDWF16(pic_processor *new_cpu, unsigned int new_opcode) : ADDWF(new_cpu,new_opcode){};
+  ADDWF16(Processor *new_cpu, unsigned int new_opcode) : ADDWF(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new ADDWF16(new_cpu,new_opcode);}
 };
 
@@ -109,9 +113,9 @@ class ADDWFC : public Register_op
 {
 public:
 
-  ADDWFC(pic_processor *new_cpu, unsigned int new_opcode);
+  ADDWFC(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new ADDWFC(new_cpu,new_opcode);}
 };
 
@@ -120,11 +124,11 @@ class ANDLW16 : public ANDLW
 {
 
 public:
-  ANDLW16(pic_processor *new_cpu, unsigned int new_opcode) :
+  ANDLW16(Processor *new_cpu, unsigned int new_opcode) :
     ANDLW(new_cpu,  new_opcode)
     {};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new ANDLW16(new_cpu,new_opcode);}
 };
 
@@ -133,9 +137,9 @@ class ANDWF16 : public ANDWF
 {
 public:
 
-  ANDWF16(pic_processor *new_cpu, unsigned int new_opcode) : ANDWF(new_cpu,new_opcode){};
+  ANDWF16(Processor *new_cpu, unsigned int new_opcode) : ANDWF(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new ANDWF16(new_cpu,new_opcode);}
 };
 
@@ -144,9 +148,9 @@ class BC : public Branching
 {
 public:
 
-  BC(pic_processor *new_cpu, unsigned int new_opcode);
+  BC(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BC(new_cpu,new_opcode);}
 
 };
@@ -156,9 +160,9 @@ class BN : public Branching
 {
 public:
 
-  BN(pic_processor *new_cpu, unsigned int new_opcode);
+  BN(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BN(new_cpu,new_opcode);}
 
 };
@@ -168,9 +172,9 @@ class BNC : public Branching
 {
 public:
 
-  BNC(pic_processor *new_cpu, unsigned int new_opcode);
+  BNC(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BNC(new_cpu,new_opcode);}
 
 };
@@ -180,9 +184,9 @@ class BNN : public Branching
 {
 public:
 
-  BNN(pic_processor *new_cpu, unsigned int new_opcode);
+  BNN(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BNN(new_cpu,new_opcode);}
 
 };
@@ -192,9 +196,9 @@ class BNOV : public Branching
 {
 public:
 
-  BNOV(pic_processor *new_cpu, unsigned int new_opcode);
+  BNOV(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BNOV(new_cpu,new_opcode);}
 
 };
@@ -204,9 +208,9 @@ class BNZ : public Branching
 {
 public:
 
-  BNZ(pic_processor *new_cpu, unsigned int new_opcode);
+  BNZ(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BNZ(new_cpu,new_opcode);}
 
 };
@@ -216,9 +220,9 @@ class BOV : public Branching
 {
 public:
 
-  BOV(pic_processor *new_cpu, unsigned int new_opcode);
+  BOV(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BOV(new_cpu,new_opcode);}
 
 };
@@ -227,13 +231,14 @@ public:
 class BRA : public instruction
 {
 public:
-  int destination;
-  unsigned int absolute_destination;
+  int destination_index;
+  unsigned int absolute_destination_index;
 
-  BRA(pic_processor *new_cpu, unsigned int new_opcode);
+  BRA(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  char *name(char *return_str);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  virtual char *name(char *,int);
+  virtual bool isBase() { return true;}
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BRA(new_cpu,new_opcode);}
 };
 
@@ -242,9 +247,9 @@ class BTG : public Bit_op
 {
 public:
 
-  BTG(pic_processor *new_cpu, unsigned int new_opcode);
+  BTG(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BTG(new_cpu,new_opcode);}
 
 
@@ -255,9 +260,9 @@ class BZ : public Branching
 {
 public:
 
-  BZ(pic_processor *new_cpu, unsigned int new_opcode);
+  BZ(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new BZ(new_cpu,new_opcode);}
 
 };
@@ -269,11 +274,11 @@ class CALL16 : public multi_word_branch
 public:
   bool fast;
 
-  CALL16(pic_processor *new_cpu, unsigned int new_opcode);
+  CALL16(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new CALL16(new_cpu,new_opcode);}
-  char *name(char *str);
+  virtual char *name(char *,int);
 
 };
 
@@ -283,9 +288,9 @@ class COMF16 : public COMF
 {
 public:
 
-  COMF16(pic_processor *new_cpu, unsigned int new_opcode) : COMF(new_cpu,new_opcode){};
+  COMF16(Processor *new_cpu, unsigned int new_opcode) : COMF(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new COMF16(new_cpu,new_opcode);}
 
 };
@@ -295,9 +300,9 @@ class CPFSEQ : public Register_op
 {
 public:
 
-  CPFSEQ(pic_processor *new_cpu, unsigned int new_opcode);
+  CPFSEQ(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new CPFSEQ(new_cpu,new_opcode);}
 
 };
@@ -307,9 +312,9 @@ class CPFSGT : public Register_op
 {
 public:
 
-  CPFSGT(pic_processor *new_cpu, unsigned int new_opcode);
+  CPFSGT(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new CPFSGT(new_cpu,new_opcode);}
 
 };
@@ -319,9 +324,9 @@ class CPFSLT : public Register_op
 {
 public:
 
-  CPFSLT(pic_processor *new_cpu, unsigned int new_opcode);
+  CPFSLT(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new CPFSLT(new_cpu,new_opcode);}
 
 };
@@ -331,9 +336,10 @@ class DAW : public instruction
 {
 public:
 
-  DAW(pic_processor *new_cpu, unsigned int new_opcode);
+  DAW(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  virtual bool isBase() { return true;}
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new DAW(new_cpu,new_opcode);}
 
 };
@@ -343,9 +349,9 @@ class DECF16 : public DECF
 {
 public:
 
-  DECF16(pic_processor *new_cpu, unsigned int new_opcode) : DECF(new_cpu,new_opcode){};
+  DECF16(Processor *new_cpu, unsigned int new_opcode) : DECF(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new DECF16(new_cpu,new_opcode);}
 
 };
@@ -355,9 +361,9 @@ class DECFSZ16 : public DECFSZ
 {
 public:
 
-  DECFSZ16(pic_processor *new_cpu, unsigned int new_opcode) : DECFSZ(new_cpu,new_opcode){};
+  DECFSZ16(Processor *new_cpu, unsigned int new_opcode) : DECFSZ(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new DECFSZ16(new_cpu,new_opcode);}
 
 };
@@ -367,9 +373,9 @@ class DCFSNZ : public Register_op
 {
 public:
 
-  DCFSNZ(pic_processor *new_cpu, unsigned int new_opcode);
+  DCFSNZ(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new DCFSNZ(new_cpu,new_opcode);}
 
 };
@@ -379,9 +385,9 @@ class GOTO16 : public multi_word_branch
 {
 public:
 
-  GOTO16(pic_processor *new_cpu, unsigned int new_opcode);
+  GOTO16(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new GOTO16(new_cpu,new_opcode);}
 };
 
@@ -391,9 +397,9 @@ class INCF16 : public INCF
 {
 public:
 
-  INCF16(pic_processor *new_cpu, unsigned int new_opcode) : INCF(new_cpu,new_opcode){};
+  INCF16(Processor *new_cpu, unsigned int new_opcode) : INCF(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new INCF16(new_cpu,new_opcode);}
 
 };
@@ -403,9 +409,9 @@ class INCFSZ16 : public INCFSZ
 {
 public:
 
-  INCFSZ16(pic_processor *new_cpu, unsigned int new_opcode) : INCFSZ(new_cpu,new_opcode){};
+  INCFSZ16(Processor *new_cpu, unsigned int new_opcode) : INCFSZ(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new INCFSZ16(new_cpu,new_opcode);}
 
 };
@@ -415,9 +421,9 @@ class INFSNZ : public Register_op
 {
 public:
 
-  INFSNZ(pic_processor *new_cpu, unsigned int new_opcode);
+  INFSNZ(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new INFSNZ(new_cpu,new_opcode);}
 
 };
@@ -428,11 +434,11 @@ class IORLW16 : public IORLW
 {
 
 public:
-  IORLW16(pic_processor *new_cpu, unsigned int new_opcode) :
+  IORLW16(Processor *new_cpu, unsigned int new_opcode) :
     IORLW(new_cpu,  new_opcode)
     {};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new IORLW16(new_cpu,new_opcode);}
 };
 
@@ -441,9 +447,9 @@ class IORWF16 : public IORWF
 {
 public:
 
-  IORWF16(pic_processor *new_cpu, unsigned int new_opcode) : IORWF(new_cpu,new_opcode){};
+  IORWF16(Processor *new_cpu, unsigned int new_opcode) : IORWF(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new IORWF16(new_cpu,new_opcode);}
 };
 
@@ -453,11 +459,11 @@ class LCALL16 : public multi_word_branch
 public:
   bool fast;
 
-  LCALL16(pic_processor *new_cpu, unsigned int new_opcode);
+  LCALL16(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new LCALL16(new_cpu,new_opcode);}
-  char *name(char *str);
+  virtual char *name(char *,int);
 
 };
 
@@ -468,12 +474,12 @@ public:
   unsigned int fsr,k;
   Indirect_Addressing *ia;
 
-  LFSR(pic_processor *new_cpu, unsigned int new_opcode);
+  LFSR(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  char *name(char *return_str);
+  virtual char *name(char *,int);
   void runtime_initialize(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new LFSR(new_cpu,new_opcode);}
 };
 
@@ -483,9 +489,9 @@ class MOVF16 : public MOVF
 {
 public:
 
-  MOVF16(pic_processor *new_cpu, unsigned int new_opcode) : MOVF(new_cpu,new_opcode){};
+  MOVF16(Processor *new_cpu, unsigned int new_opcode) : MOVF(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MOVF16(new_cpu,new_opcode);}
 
 };
@@ -496,12 +502,12 @@ class MOVFF : public multi_word_instruction
 public:
   unsigned int source,destination;
 
-  MOVFF(pic_processor *new_cpu, unsigned int new_opcode);
+  MOVFF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  char *name(char *return_str);
+  virtual char *name(char *,int);
   void runtime_initialize(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MOVFF(new_cpu,new_opcode);}
 };
 
@@ -512,12 +518,12 @@ class MOVFP : public multi_word_instruction
 public:
   unsigned int source,destination;
 
-  MOVFP(pic_processor *new_cpu, unsigned int new_opcode);
+  MOVFP(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  char *name(char *return_str);
+  virtual char *name(char *,int);
   void runtime_initialize(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MOVFP(new_cpu,new_opcode);}
 };
 
@@ -527,9 +533,9 @@ public:
 class MOVLB : public Literal_op
 {
 public:
-  MOVLB(pic_processor *new_cpu, unsigned int new_opcode);
+  MOVLB(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MOVLB(new_cpu,new_opcode);}
 
 };
@@ -539,9 +545,9 @@ public:
 class MOVLR : public Literal_op
 {
 public:
-  MOVLR(pic_processor *new_cpu, unsigned int new_opcode);
+  MOVLR(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MOVLR(new_cpu,new_opcode);}
 
 };
@@ -552,12 +558,12 @@ class MOVPF : public multi_word_instruction
 public:
   unsigned int source,destination;
 
-  MOVPF(pic_processor *new_cpu, unsigned int new_opcode);
+  MOVPF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  char *name(char *return_str);
+  virtual char *name(char *,int);
   void runtime_initialize(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MOVPF(new_cpu,new_opcode);}
 };
 
@@ -567,22 +573,36 @@ class MOVWF16 : public MOVWF
 {
 public:
 
-  MOVWF16(pic_processor *new_cpu, unsigned int new_opcode) : MOVWF(new_cpu,new_opcode){};
+  MOVWF16(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MOVWF16(new_cpu,new_opcode);}
 
 };
 
 //---------------------------------------------------------
 
+class MOVWF16a : public MOVWF
+{
+public:
+
+  MOVWF16a(Processor *new_cpu, unsigned int new_opcode);
+  virtual void execute(void);
+
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
+    {return new MOVWF16a(new_cpu,new_opcode);}
+
+};
+
+//---------------------------------------------------------
+
 class MULLW : public Literal_op
 {
 public:
-  MULLW(pic_processor *new_cpu, unsigned int new_opcode);
+  MULLW(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MULLW(new_cpu,new_opcode);}
 
 };
@@ -593,9 +613,9 @@ class MULWF : public Register_op
 {
 public:
 
-  MULWF(pic_processor *new_cpu, unsigned int new_opcode);
+  MULWF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new MULWF(new_cpu,new_opcode);}
 
 };
@@ -605,9 +625,9 @@ class NEGF : public Register_op
 {
 public:
 
-  NEGF(pic_processor *new_cpu, unsigned int new_opcode);
+  NEGF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new NEGF(new_cpu,new_opcode);}
 
 };
@@ -617,9 +637,9 @@ class NEGW : public Register_op
 {
 public:
 
-  NEGW(pic_processor *new_cpu, unsigned int new_opcode);
+  NEGW(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new NEGW(new_cpu,new_opcode);}
 
 };
@@ -629,9 +649,10 @@ class POP : public instruction
 {
 public:
 
-  POP(pic_processor *new_cpu, unsigned int new_opcode);
+  POP(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  virtual bool isBase() { return true;}
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new POP(new_cpu,new_opcode);}
 
 };
@@ -641,9 +662,10 @@ class PUSH : public instruction
 {
 public:
 
-  PUSH(pic_processor *new_cpu, unsigned int new_opcode);
+  PUSH(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  virtual bool isBase() { return true;}
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new PUSH(new_cpu,new_opcode);}
 
 };
@@ -652,13 +674,14 @@ public:
 class RCALL : public instruction
 {
 public:
-  int destination;
-  unsigned int absolute_destination;
+  int destination_index;
+  unsigned int absolute_destination_index;
 
-  RCALL(pic_processor *new_cpu, unsigned int new_opcode);
+  RCALL(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  char *name(char *return_str);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  virtual char *name(char *,int);
+  virtual bool isBase() { return true;}
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new RCALL(new_cpu,new_opcode);}
 };
 
@@ -667,9 +690,10 @@ class RESET : public instruction
 {
 public:
 
-  RESET(pic_processor *new_cpu, unsigned int new_opcode);
+  RESET(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  virtual bool isBase() { return true;}
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new RESET(new_cpu,new_opcode);}
 
 };
@@ -682,15 +706,15 @@ class RETFIE16 : public RETFIE
 public:
   bool fast;
 
-  RETFIE16(pic_processor *new_cpu, unsigned int new_opcode) : 
+  RETFIE16(Processor *new_cpu, unsigned int new_opcode) : 
     RETFIE(new_cpu,new_opcode)
     {
       fast = (new_opcode & 1);
     };
   virtual void execute(void);
-  virtual char *name(char *);
+  virtual char *name(char *,int);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new RETFIE16(new_cpu,new_opcode);}
 
 };
@@ -702,15 +726,15 @@ class RETURN16 : public RETURN
 public:
   bool fast;
 
-  RETURN16(pic_processor *new_cpu, unsigned int new_opcode) : 
+  RETURN16(Processor *new_cpu, unsigned int new_opcode) : 
     RETURN(new_cpu,new_opcode)
     {
       fast = (new_opcode & 1);
     };
   virtual void execute(void);
-  virtual char *name(char *);
+  virtual char *name(char *,int);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new RETURN16(new_cpu,new_opcode);}
 
 };
@@ -721,10 +745,10 @@ class RLCF : public Register_op
 {
 public:
 
-  RLCF(pic_processor *new_cpu, unsigned int new_opcode);
+  RLCF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new RLCF(new_cpu,new_opcode);}
 
 };
@@ -734,10 +758,10 @@ class RLNCF : public Register_op
 {
 public:
 
-  RLNCF(pic_processor *new_cpu, unsigned int new_opcode);
+  RLNCF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new RLNCF(new_cpu,new_opcode);}
 
 };
@@ -748,10 +772,10 @@ class RRCF : public Register_op
 {
 public:
 
-  RRCF(pic_processor *new_cpu, unsigned int new_opcode);
+  RRCF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new RRCF(new_cpu,new_opcode);}
 
 };
@@ -761,10 +785,10 @@ class RRNCF : public Register_op
 {
 public:
 
-  RRNCF(pic_processor *new_cpu, unsigned int new_opcode);
+  RRNCF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new RRNCF(new_cpu,new_opcode);}
 
 };
@@ -774,9 +798,9 @@ class SETF : public Register_op
 {
 public:
 
-  SETF(pic_processor *new_cpu, unsigned int new_opcode);
+  SETF(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new SETF(new_cpu,new_opcode);}
 
 };
@@ -786,11 +810,11 @@ class SLEEP16 : public SLEEP
 {
 public:
 
-  SLEEP16(pic_processor *new_cpu, unsigned int new_opcode) : 
+  SLEEP16(Processor *new_cpu, unsigned int new_opcode) : 
     SLEEP(new_cpu,new_opcode) { };
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new SLEEP16(new_cpu,new_opcode);}
 
 };
@@ -800,10 +824,10 @@ class SUBFWB : public Register_op
 {
 public:
 
-  SUBFWB(pic_processor *new_cpu, unsigned int new_opcode);
+  SUBFWB(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new SUBFWB(new_cpu,new_opcode);}
 
 };
@@ -815,11 +839,11 @@ class SUBLW16 : public SUBLW
 
 public:
 
-  SUBLW16(pic_processor *new_cpu, unsigned int new_opcode) : 
+  SUBLW16(Processor *new_cpu, unsigned int new_opcode) : 
     SUBLW(new_cpu,new_opcode) { };
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new SUBLW16(new_cpu,new_opcode);}
 
 };
@@ -831,11 +855,11 @@ class SUBWF16 : public SUBWF
 
 public:
 
-  SUBWF16(pic_processor *new_cpu, unsigned int new_opcode) : 
+  SUBWF16(Processor *new_cpu, unsigned int new_opcode) : 
     SUBWF(new_cpu,new_opcode) { };
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new SUBWF16(new_cpu,new_opcode);}
 
 };
@@ -845,10 +869,10 @@ class SUBWFB : public Register_op
 {
 public:
 
-  SUBWFB(pic_processor *new_cpu, unsigned int new_opcode);
+  SUBWFB(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new SUBWFB(new_cpu,new_opcode);}
 
 };
@@ -858,11 +882,12 @@ class TBLRD : public instruction
 {
 public:
 
-  TBLRD(pic_processor *new_cpu, unsigned int new_opcode);
+  TBLRD(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  virtual char *name(char *);
+  virtual char *name(char *,int);
+  virtual bool isBase() { return true;}
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new TBLRD(new_cpu,new_opcode);}
 
 };
@@ -872,11 +897,12 @@ class TBLWT : public instruction
 {
 public:
 
-  TBLWT(pic_processor *new_cpu, unsigned int new_opcode);
+  TBLWT(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  virtual char *name(char *);
+  virtual char *name(char *,int);
+  virtual bool isBase() { return true;}
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new TBLWT(new_cpu,new_opcode);}
 
 };
@@ -886,11 +912,12 @@ class TLRD : public instruction
 {
 public:
 
-  TLRD(pic_processor *new_cpu, unsigned int new_opcode);
+  TLRD(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  virtual char *name(char *);
+  virtual char *name(char *,int);
+  virtual bool isBase() { return true;}
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new TLRD(new_cpu,new_opcode);}
 
 };
@@ -900,11 +927,12 @@ class TLWT : public instruction
 {
 public:
 
-  TLWT(pic_processor *new_cpu, unsigned int new_opcode);
+  TLWT(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
-  virtual char *name(char *);
+  virtual char *name(char *,int);
+  virtual bool isBase() { return true;}
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new TLWT(new_cpu,new_opcode);}
 
 };
@@ -914,10 +942,10 @@ class TSTFSZ : public Register_op
 {
 public:
 
-  TSTFSZ(pic_processor *new_cpu, unsigned int new_opcode);
+  TSTFSZ(Processor *new_cpu, unsigned int new_opcode);
   virtual void execute(void);
 
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new TSTFSZ(new_cpu,new_opcode);}
 
 };
@@ -927,11 +955,11 @@ class XORLW16 : public XORLW
 {
 
 public:
-  XORLW16(pic_processor *new_cpu, unsigned int new_opcode) :
+  XORLW16(Processor *new_cpu, unsigned int new_opcode) :
     XORLW(new_cpu,  new_opcode)
     {};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new XORLW16(new_cpu,new_opcode);}
 };
 
@@ -940,9 +968,9 @@ class XORWF16 : public XORWF
 {
 public:
 
-  XORWF16(pic_processor *new_cpu, unsigned int new_opcode) : XORWF(new_cpu,new_opcode){};
+  XORWF16(Processor *new_cpu, unsigned int new_opcode) : XORWF(new_cpu,new_opcode){};
   virtual void execute(void);
-  static instruction *construct(pic_processor *new_cpu, unsigned int new_opcode)
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode)
     {return new XORWF16(new_cpu,new_opcode);}
 };
 
