@@ -20,8 +20,8 @@ Boston, MA 02111-1307, USA.  */
 
 
 #include <stdio.h>
-#include <iostream.h>
-#include <iomanip.h>
+#include <iostream>
+#include <iomanip>
 
 
 #include "../config.h"
@@ -38,23 +38,68 @@ Boston, MA 02111-1307, USA.  */
 //--------------------------------------------------
 TMR0::TMR0(void)
 {
-  break_point = 0;
-  value=0;
+  value.put(0);
   synchronized_cycle=0;
   future_cycle=0;
   last_cycle=0;
+  state=0;      // start disabled (will change to enabled by other init code)
   prescale=1;
+  m_bLastClockedState=false;
   new_name("tmr0");
 }
 
-void TMR0::start(int restart_value, int sync=0)
+//------------------------------------------------------------------------
+// setSinkState
+//
+// Called when the I/O pin driving TMR0 changes states.
+
+void TMR0::setSinkState(char new3State)
+{
+  bool bNewState = new3State == '1';
+
+  if (m_bLastClockedState != bNewState) {
+    m_bLastClockedState = bNewState;
+    //printf("TMR0::setSinkState:%d cs:%d se:%d\n",bNewState,get_t0cs(),get_t0se());
+    if (get_t0cs() && bNewState == get_t0se())
+      increment();
+  }
+}
+
+void TMR0::set_cpu(Processor *new_cpu, PortRegister *reg, unsigned int pin)
+{
+  cpu = new_cpu;
+  reg->addSink(this,pin);
+}
+
+//------------------------------------------------------------
+// Stop the tmr.
+//
+void TMR0::stop(void)
 {
 
-  value = restart_value;
+  // If tmr0 is running:
+
+  if (state & 1) {
+
+    // refresh the current value.
+    get_value();
+
+    state &= (~1);      // the timer is disabled.
+    clear_break();
+
+  }
+}
+
+void TMR0::start(int restart_value, int sync)
+{
+
+  state |= 1;          // the timer is on
+
+  value.put(restart_value&0xff);
   if(verbose)
     cout << "TMRO::start\n";
 
-  old_option = cpu->option_reg.value;
+  old_option = cpu_pic->option_reg.value.get();
 
   prescale = 1 << get_prescale();
   prescale_counter = prescale;
@@ -65,18 +110,18 @@ void TMR0::start(int restart_value, int sync=0)
 
   } else {
 
-    synchronized_cycle = cpu->cycles.value + sync;
+    synchronized_cycle = get_cycles().value + sync;
   
-    last_cycle = value * prescale;
+    last_cycle = (restart_value % max_counts()) * prescale;
     last_cycle = synchronized_cycle - last_cycle;
 
     guint64 fc = last_cycle + max_counts() * prescale;
 
 
     if(future_cycle)
-      cpu->cycles.reassign_break(future_cycle, fc, this);
+      get_cycles().reassign_break(future_cycle, fc, this);
     else
-      cpu->cycles.set_break(fc, this);
+      get_cycles().set_break(fc, this);
 
     future_cycle = fc;
 
@@ -95,6 +140,9 @@ void TMR0::clear_break(void)
   if(verbose)
     cout << "TMR0 break was cleared\n";
 
+  if (future_cycle)
+    get_cycles().clear_break(this);
+
   future_cycle = 0;
   last_cycle = 0;
 
@@ -102,52 +150,62 @@ void TMR0::clear_break(void)
 
 unsigned int TMR0::get_prescale(void)
 {
-  return (cpu->option_reg.get_psa() ? 0 : (1+cpu->option_reg.get_prescale()));
+  return (cpu_pic->option_reg.get_psa() ? 0 : (1+cpu_pic->option_reg.get_prescale()));
 }
 
-// %%%FIX ME%%% 
 void TMR0::increment(void)
 {
-  //  cout << "TMR0 increment because of external clock ";
+  //cout << "TMR0 increment because of external clock ";
+
+  if((state & 1) == 0)
+    return;
 
   if(--prescale_counter == 0)
     {
+      trace.raw(write_trace.get() | value.get());
       prescale_counter = prescale;
-      if(++value == 256)
+      if(value.get() == 255)
 	{
-	  value = 0;
+	  //cout << "TMR0 rollover because of external clock ";
+	  value.put(0);
 	  set_t0if();
 
 	}
-      trace.register_write(address,value);
+      else
+	value.put(value.get() + 1);
     }
-  //  cout << value << '\n';
+  //  cout << "TMR0 value ="<<value.get() << '\n';
+}
+
+
+void TMR0::put_value(unsigned int new_value)
+{
+  value.put(new_value & 0xff);
+
+  // If tmr0 is enabled, then start it up.
+  if(state & 1)
+    start(new_value,2);
 }
 
 void TMR0::put(unsigned int new_value)
 {
-  if(get_t0cs())
-    {
-      cout << "TMR0::put external clock...\n";
-    }
-
-  start(new_value,2);
-
-  trace.register_write(address,value);
-
+  trace.raw(write_trace.get() | value.get());
+  put_value(new_value);
 }
 
 unsigned int TMR0::get_value(void)
 {
   // If the TMR0 is being read immediately after being written, then
   // it hasn't had enough time to synchronize with the PIC's clock.
-  if(cpu->cycles.value <= synchronized_cycle)
-    return value;
+  if(get_cycles().value <= synchronized_cycle)
+    return value.get();
 
-  if(get_t0cs())
-    return(value);
+  // If we're running off the external clock or the tmr is disabled
+  // then just return the register value.
+  if(get_t0cs() || ((state & 1)==0))
+    return(value.get());
 
-  int new_value = (cpu->cycles.value - last_cycle)/ prescale;
+  int new_value = (int )((get_cycles().value - last_cycle)/ prescale);
 
   //  if(new_value == 256) {
     // tmr0 is about to roll over. However, the user code
@@ -161,7 +219,7 @@ unsigned int TMR0::get_value(void)
   if (new_value > 255)
     {
       cout << "TMR0: bug TMR0 is larger than 255...\n";
-      cout << "cpu->cycles.value = " << cpu->cycles.value <<
+      cout << "cycles.value = " << get_cycles().value <<
 	"  last_cycle = " << last_cycle <<
 	"  prescale = "  << prescale << 
 	"  calculated value = " << new_value << '\n';
@@ -171,28 +229,28 @@ unsigned int TMR0::get_value(void)
       // let's just go ahead and reset the logic.
       new_value &= 0xff;
       last_cycle = new_value*prescale;
-      last_cycle = cpu->cycles.value - last_cycle;
+      last_cycle = get_cycles().value - last_cycle;
       synchronized_cycle = last_cycle;
     }
 
-  value = new_value;
-  return(value);
+  value.put(new_value);
+  return(value.get());
   
 }
 
 unsigned int TMR0::get(void)
 {
-  value = get_value();
-  trace.register_read(address, value);
-  return value;
+  value.put(get_value());
+  trace.raw(read_trace.get() | value.get());
+  return value.get();
 }
 void TMR0::new_prescale(void)
 {
   //cout << "tmr0 new_prescale\n";
 
-  int new_value;
+  unsigned int new_value;
 
-  int option_diff = old_option ^ cpu->option_reg.value;
+  int option_diff = old_option ^ cpu_pic->option_reg.value.get();
 
   old_option ^= option_diff;   // save old option value. ( (a^b) ^b = a)
 
@@ -201,35 +259,36 @@ void TMR0::new_prescale(void)
     if(verbose)
       cout << "T0CS has changed to ";
 
-    if(cpu->option_reg.get_t0cs()) {
+    if(cpu_pic->option_reg.get_t0cs()) {
       // External clock
       if(verbose)
 	cout << "external clock\n";
-      cpu->cycles.clear_break(future_cycle);
+      get_cycles().clear_break(future_cycle);
+      future_cycle = 0;
     } else {
+      // Internal Clock
       if(verbose)
 	cout << "internal clock\n";
-      // Internal Clock
-      start(value);
     }
+    start(value.get());
 
   } else {
 
-    if(cpu->option_reg.get_t0cs()) {
+    if(get_t0cs() || ((state & 1)==0)) {
       prescale = 1 << get_prescale();
       prescale_counter = prescale;
 
     } else {
 
-      if(last_cycle < cpu->cycles.value)
-	new_value = (cpu->cycles.value - last_cycle)/prescale;
-      else
-	new_value = 0;
+    if(last_cycle < (gint64)get_cycles().value)
+	  new_value = (unsigned int)((get_cycles().value - last_cycle)/prescale);
+    else
+	  new_value = 0;
 
       if(new_value>=max_counts()) {
 	cout << "TMR0 bug (new_prescale): exceeded max count"<< max_counts() <<'\n';
 	cout << "   last_cycle = 0x" << hex << last_cycle << endl;
-	cout << "   cpu cycle = 0x" << hex << (cpu->cycles.value) << endl;
+	cout << "   cpu cycle = 0x" << hex << (get_cycles().value) << endl;
 
 	cout << "   prescale = 0x" << hex << prescale << endl;
 
@@ -238,7 +297,7 @@ void TMR0::new_prescale(void)
 
       // Get the current value of TMR0
 
-      // cout << "cycles " << cpu->cycles.value  << " old prescale " << prescale;
+      // cout << "cycles " << cycles.value  << " old prescale " << prescale;
 
       prescale = 1 << get_prescale();
       prescale_counter = prescale;
@@ -249,32 +308,37 @@ void TMR0::new_prescale(void)
       // new prescale all along. Recall, 'last_cycle' records the value of the cpu's
       // cycle counter when tmr0 last rolled over.
 
-      last_cycle = value * prescale;
+      last_cycle = value.get() * prescale;
       last_cycle = synchronized_cycle - last_cycle;
 
       // cout << " effective last_cycle " << last_cycle << '\n';
 
-      if(cpu->cycles.value <= synchronized_cycle)
-	last_cycle += (synchronized_cycle - cpu->cycles.value);
+      if(get_cycles().value <= synchronized_cycle)
+	last_cycle += (synchronized_cycle - get_cycles().value);
 
       guint64 fc = last_cycle + max_counts() * prescale;
 
       // cout << "moving break from " << future_cycle << " to " << fc << '\n';
 
-      cpu->cycles.reassign_break(future_cycle, fc, this);
+      get_cycles().reassign_break(future_cycle, fc, this);
 
       future_cycle = fc;
     }
   }
 }
 
-unsigned int TMR0::get_t0cs(void)
+bool TMR0::get_t0cs(void)
 {
-  return cpu->option_reg.get_t0cs();
+  return cpu_pic->option_reg.get_t0cs() != 0;
 }
+bool TMR0::get_t0se(void)
+{
+  return cpu_pic->option_reg.get_t0se() != 0;
+}
+
 void TMR0::set_t0if(void)
 {
-  if(cpu->base_isa() == _14BIT_PROCESSOR_)
+  if(cpu_pic->base_isa() == _14BIT_PROCESSOR_)
     {
       cpu14->intcon->set_t0if();
     }
@@ -288,12 +352,12 @@ void TMR0::new_clock_source(void)
   if(get_t0cs())
     {
       //cout << "external\n";
-      //      cpu->cycles.
+      //      cycles.
     }
   else
     {
       //cout << "internal\n";
-      put(value);    // let TMR0::put() set a cycle counter break point
+      put(value.get());    // let TMR0::put() set a cycle counter break point
     }
 #endif
 }
@@ -305,7 +369,12 @@ void TMR0::new_clock_source(void)
 void TMR0::callback(void)
 {
 
-  //cout<<"TMR0 rollover: " << hex << cpu->cycles.value << '\n';
+  //cout<<"TMR0 rollover: " << hex << cycles.value << '\n';
+
+  if((state & 1) == 0) {
+
+    cout << "TMR0 callback ignored because timer is disabled\n";
+  }
 
   // If tmr0 is being clocked by the external clock, then at some point
   // the simulate code must have switched from the internal clock to
@@ -316,11 +385,11 @@ void TMR0::callback(void)
       return;
     }
 
-  value = 0;
-  synchronized_cycle = cpu->cycles.value;
+  value.put(0);
+  synchronized_cycle = get_cycles().value;
   last_cycle = synchronized_cycle;
   future_cycle = last_cycle + max_counts()*prescale;
-  cpu->cycles.set_break(future_cycle, this);
+  get_cycles().set_break(future_cycle, this);
   set_t0if();
 }
 
@@ -328,8 +397,7 @@ void  TMR0::reset(RESET_TYPE r)
 {
 
   switch(r) {
-  POR_RESET:
-    cout << "TMR0::reset - por\n";
+  case POR_RESET:
     value = por_value;
     break;
   default:
