@@ -19,14 +19,89 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 #include <stdio.h>
-#include <iostream.h>
+#include <iostream>
 
 #include "../config.h"
+#include "stimuli.h"
 #include "uart.h"
 #include "14bit-processors.h"
 #include "14bit-tmrs.h"
 
-#include "xref.h"
+//#define DEBUG
+#if defined(DEBUG)
+#define Dprintf(arg) {printf("%s:%d-%s() ",__FILE__,__LINE__,__FUNCTION__); printf arg; }
+#else
+#define Dprintf(arg) {}
+#endif
+
+//--------------------------------------------------
+// 
+//--------------------------------------------------
+class TXSignalSource : public SignalControl
+{
+public:
+  TXSignalSource(_TXSTA *_txsta)
+    : m_txsta(_txsta)
+  {
+    assert(m_txsta);
+  }
+  char getState()
+  {
+    return m_txsta->getState();
+  }
+private:
+  _TXSTA *m_txsta;
+};
+
+//--------------------------------------------------
+//
+//--------------------------------------------------
+
+class RXSignalSink : public SignalSink
+{
+public:
+  RXSignalSink(_RCSTA *_rcsta)
+    : m_rcsta(_rcsta)
+  {
+    assert(_rcsta);
+  }
+
+  void setSinkState(char new3State)
+  {
+    m_rcsta->setState(new3State);
+  }
+private:
+  _RCSTA *m_rcsta;
+};
+
+//-----------------------------------------------------------
+_RCSTA::_RCSTA()
+  : rcreg(0), spbrg(0), txsta(0), m_PinModule(0), m_sink(0), m_cRxState('?')
+{
+}
+
+//-----------------------------------------------------------
+_TXSTA::_TXSTA()
+  : txreg(0), spbrg(0), m_PinModule(0),  m_source(0), m_cTxState('?')
+{
+}
+
+//-----------------------------------------------------------
+_RCREG::_RCREG()
+ : m_rcsta(0), pir_set(0)
+{
+}
+
+_TXREG::_TXREG()
+ : m_txsta(0), pir_set(0)
+{
+}
+
+
+_SPBRG::_SPBRG()
+  : txsta(0), rcsta(0)
+{
+}
 
 //-----------------------------------------------------------
 // TXREG - USART Transmit Register
@@ -35,26 +110,25 @@ Boston, MA 02111-1307, USA.  */
 void _TXREG::put(unsigned int new_value)
 {
 
-  value = new_value & 0xff;
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value & 0xff);
 
-  if(verbose)
-    cout << "txreg just got a new value\n";
+  Dprintf(("txreg just got a new value:0x%x\n",new_value));
 
   // The transmit register has data,
   // so clear the TXIF flag
 
   full();
 
-  if( (txsta->value & (_TXSTA::TRMT | _TXSTA::TXEN)) == (_TXSTA::TRMT | _TXSTA::TXEN))
+  if(m_txsta &&
+     ( (m_txsta->value.get() & (_TXSTA::TRMT | _TXSTA::TXEN)) == (_TXSTA::TRMT | _TXSTA::TXEN)))
     {
       // If the transmit buffer is empty and the transmitter is enabled
       // then transmit this new data now...
 
-      txsta->start_transmitting();
+      m_txsta->start_transmitting();
     }
 
-  trace.register_write(address,value);
-
 }
 
 void _TXREG::put_value(unsigned int new_value)
@@ -62,8 +136,34 @@ void _TXREG::put_value(unsigned int new_value)
 
   put(new_value);
 
-  if(xref)
-    xref->update();
+  update();
+}
+
+//-----------------------------------------------------------
+// TXSTA - setIOpin - assign the I/O pin associated with the
+// the transmitter.
+
+
+void _TXSTA::setIOpin(PinModule *newPinModule)
+{
+  if (!m_source) {
+    m_source = new TXSignalSource(this);
+    m_PinModule = newPinModule;
+  }
+
+}
+//-----------------------------------------------------------
+// TXSTA - putTXState - update the state of the TX output pin
+//
+
+void _TXSTA::putTXState(char newTXState)
+{
+
+  m_cTxState = newTXState;
+
+  if (m_PinModule)
+    m_PinModule->updatePinModule();
+
 }
 
 //-----------------------------------------------------------
@@ -74,26 +174,26 @@ void _TXSTA::put_value(unsigned int new_value)
 
   put(new_value);
 
-  if(xref)
-    xref->update();
+  update();
 
 }
 
 void _TXSTA::put(unsigned int new_value)
 {
 
-  unsigned int old_value = value;
+  unsigned int old_value = value.get();
+
+  trace.raw(write_trace.get() | value.get());
 
   // The TRMT bit is controlled entirely by hardware.
   // It is high if the TSR has any data.
 
-  value = (new_value & ~TRMT) | ( (bit_count) ? 0 : TRMT);
+  value.put((new_value & ~TRMT) | ( (bit_count) ? 0 : TRMT));
 
-  if(verbose)
-    cout << "TXSTA::put 0x" << value << '\n';
+  Dprintf((" TXSTA value=0x%x\n",value.get()));
 
 
-  if( (old_value ^ value) & TXEN) {
+  if( (old_value ^ value.get()) & TXEN) {
 
     // The TXEN bit has changed states.
     //
@@ -103,26 +203,46 @@ void _TXSTA::put(unsigned int new_value)
     // If the transmitter is being disabled, then abort any
     // transmission.
 
-    if(value & TXEN) {
-      if(!txreg->is_empty())
-	start_transmitting();
-    } else 
+    if(value.get() & TXEN) {
+      Dprintf(("TXSTA - enabling transmitter\n"));
+      if (m_PinModule)
+	m_PinModule->setSource(m_source);
+      if(txreg) {
+	txreg->empty();
+#if 0
+	if(txreg->is_empty()) {
+	  txreg->empty();
+	} else {
+          Dprintf(("start_transmitting\n"));
+	  start_transmitting();
+	}
+#endif
+      }
+    } else {
       stop_transmitting();
+      if (m_PinModule)
+	m_PinModule->setSource(0);
+
+    }
   }
+}
+//------------------------------------------------------------
+//
+char _TXSTA::getState()
+{
 
-  trace.register_write(address,value);
-
+  return m_cTxState;
 }
 
-// _TXSTA::stop_transmitting(void)
+
+// _TXSTA::stop_transmitting()
 //
-void _TXSTA::stop_transmitting(void)
+void _TXSTA::stop_transmitting()
 {
-  if(verbose)
-    cout << "stopping a USART transmission\n";
+  Dprintf(("stopping a USART transmission\n"));
 
   bit_count = 0;
-  value |= TRMT;
+  value.put(value.get() | TRMT);
 
   // It's not clear from the documentation as to what happens
   // to the TXIF when we are aborting a transmission. According
@@ -145,10 +265,9 @@ void _TXSTA::stop_transmitting(void)
 
 }
 
-void _TXSTA::start_transmitting(void)
+void _TXSTA::start_transmitting()
 {
-  if(verbose)
-    cout << "starting a USART transmission\n";
+  Dprintf(("starting a USART transmission\n"));
 
   // Build the serial byte that's about to be transmitted.
   // I doubt the pic really does this way, but gpsim builds
@@ -172,17 +291,19 @@ void _TXSTA::start_transmitting(void)
 
   // The start bit, which is always low, occupies bit position
   // zero. The next 8 bits come from the txreg.
+  if(!txreg)
+    return;
 
-  tsr = txreg->value << 1;
+  tsr = txreg->value.get() << 1;
 
   // Is this a 9-bit data transmission?
-  if(value & TX9)
+  if(value.get() & TX9)
     {
       // Copy the stop bit and the 9th data bit to the tsr.
       // (See note above for the reason why two stop bits 
       // are appended to the packet.)
 
-      tsr |= ( (value & TX9D) ? (7<<9) : (6<<9));
+      tsr |= ( (value.get() & TX9D) ? (7<<9) : (6<<9));
       bit_count = 12;  // 1 start, 9 data, 1 stop , 1 delay
     }
   else
@@ -198,13 +319,13 @@ void _TXSTA::start_transmitting(void)
   // Set a callback breakpoint at the next SPBRG edge
 
   if(cpu)
-    cpu->cycles.set_break(spbrg->get_cpu_cycle(1), this);
+    get_cycles().set_break(spbrg->get_cpu_cycle(1), this);
 
   // The TSR now has data, so clear the Transmit Shift 
   // Register Status bit.
 
-  value &= ~TRMT;
-  trace.register_write(address,value);
+  trace.raw(write_trace.get() | value.get());
+  value.put(value.get() & ~TRMT);
 
   // Tell the TXREG that its data has been consumed.
 
@@ -212,27 +333,27 @@ void _TXSTA::start_transmitting(void)
 
 }
 
-void _TXSTA::transmit_a_bit(void)
+void _TXSTA::transmit_a_bit()
 {
 
 
-  if(bit_count)
-    {
+  if(bit_count) {
 
-      //cout << "Transmit bit #" << bit_count << ": " << (tsr&1) << '\n';
-      txpin->put_state(tsr&1);
+    Dprintf(("Transmit bit #%x: bit val:%d time:0x%llx\n",bit_count, (tsr&1),get_cycles().value));
 
-      tsr >>= 1;
+    putTXState(tsr&1 ? '1' : '0');
 
-      --bit_count;
+    tsr >>= 1;
 
-    }
+    --bit_count;
+
+  }
 
 }
 
-void _TXSTA::callback(void)
+void _TXSTA::callback()
 {
-
+  Dprintf(("TXSTA callback - time:%llx\n",get_cycles().value));
 
   transmit_a_bit();
 
@@ -245,8 +366,8 @@ void _TXSTA::callback(void)
     // (See the note above about the 'extra' stop bit that was stuffed
     // into the tsr register. 
 
-    if(txreg->is_empty())
-      value |= TRMT;
+    if(txreg && txreg->is_empty())
+      value.put(value.get() | TRMT);
     else
       start_transmitting();
 
@@ -255,12 +376,41 @@ void _TXSTA::callback(void)
     // data in the tsr that needs to be sent.
 
     if(cpu) {
-      cpu->cycles.set_break(spbrg->get_cpu_cycle(1),this);
+      get_cycles().set_break(spbrg->get_cpu_cycle(1),this);
     }
   }
 
 }
 
+void _TXSTA::callback_print()
+{
+  cout << "TXSTA " << name() << " CallBack ID " << CallBackID << '\n';
+}
+
+bool _TXREG::is_empty()
+{
+  if (pir_set)
+    return(pir_set->get_txif());
+  return true;
+}
+void _TXREG::empty()
+{
+  Dprintf(("txreg::empty - setting TXIF\n"));
+  if (pir_set)
+    pir_set->set_txif();
+
+}
+void _TXREG::full()
+{
+  Dprintf(("txreg::full - clearing TXIF\n"));
+  if(pir_set)
+    pir_set->clear_txif();
+}
+void _TXREG::assign_pir_set(PIR_SET *new_pir_set)
+{
+  pir_set = new_pir_set;
+}
+
 //-----------------------------------------------------------
 // Receiver portion of the USART
 //-----------------------------------------------------------
@@ -292,16 +442,19 @@ void _RCSTA::put(unsigned int new_value)
 {
   unsigned int diff;
 
-  diff = new_value ^ value;
-  value = ( value & (RX9D | OERR | FERR) )   |  (new_value & ~(RX9D | OERR | FERR));
+  diff = new_value ^ value.get();
+  trace.raw(write_trace.get() | value.get());
+  value.put( ( value.get() & (RX9D | OERR | FERR) )   |  (new_value & ~(RX9D | OERR | FERR)));
 
+  if(!txsta || !txsta->txreg)
+    return;
   // First check whether or not the serial port is being enabled
   if(diff & SPEN) {
 
-    if(value & SPEN) {
+    if(value.get() & SPEN) {
       spbrg->start();
       // Make the tx line high when the serial port is enabled.
-      txsta->txpin->put_state(1);
+      txsta->putTXState('1');
       txsta->txreg->empty();
     } else {
 
@@ -316,10 +469,10 @@ void _RCSTA::put(unsigned int new_value)
 
   }
 
-  if(!(txsta->value & _TXSTA::SYNC)) {
+  if(!(txsta->value.get() & _TXSTA::SYNC)) {
 
     // Asynchronous receive.
-    if( (value & (SPEN | CREN)) == (SPEN | CREN) ) {
+    if( (value.get() & (SPEN | CREN)) == (SPEN | CREN) ) {
 
       // The receiver is enabled. Now check to see if that just happened
       
@@ -329,7 +482,8 @@ void _RCSTA::put(unsigned int new_value)
 	start_receiving();
 
 	// If the rx line is low, then go ahead and start receiving now.
-	if(!uart_port->get_bit(rx_bit))
+	//if(uart_port && !uart_port->get_bit(rx_bit))
+	if (m_cRxState == '0' || m_cRxState == 'w')
 	  receive_start_bit();
       }
 
@@ -342,8 +496,6 @@ void _RCSTA::put(unsigned int new_value)
   } else
     cout << "not doing syncronous receptions yet\n";
 
-  trace.register_write(address,value);
-  
 }
 
 void _RCSTA::put_value(unsigned int new_value)
@@ -351,10 +503,41 @@ void _RCSTA::put_value(unsigned int new_value)
 
   put(new_value);
 
-  if(xref)
-    xref->update();
+  update();
 }
 
+//-----------------------------------------------------------
+// RCSTA - setIOpin - assign the I/O pin associated with the
+// the receiver.
+
+
+void _RCSTA::setIOpin(PinModule *newPinModule)
+{
+  if (!m_sink) {
+    m_sink = new RXSignalSink(this);
+    m_PinModule = newPinModule;
+    if (m_PinModule)
+      m_PinModule->addSink(m_sink);
+  }
+
+}
+
+//-----------------------------------------------------------
+// RCSTA - setState
+// This gets called whenever there's a change detected on the RX pin.
+// The usart is only interested in those changes when it is waiting
+// for the start bit. Otherwise, the rcsta callback function will sample
+// the rx pin (if we're receiving).
+
+
+void _RCSTA::setState(char new_RxState)
+{
+  m_cRxState = new_RxState;
+
+  if( (state == RCSTA_WAITING_FOR_START) && (m_cRxState =='0' || m_cRxState=='w'))
+    receive_start_bit();
+
+}
 //-----------------------------------------------------------
 // RCSTA::receive_a_bit(unsigned int bit)
 //
@@ -368,50 +551,59 @@ void _RCSTA::put_value(unsigned int new_value)
 void _RCSTA::receive_a_bit(unsigned int bit)
 {
 
-  if(bit_count)
-    {
+  // If we're waiting for the start bit and this isn't it then
+  // we don't need to look any further
+  Dprintf(("receive_a_bit state:%d bit:%d time:0x%llx\n",state,bit,get_cycles().value));
 
-      // If we're waiting for the start bit and this isn't it then
-      // we don't need to look any further
-      if( (state == RCSTA_WAITING_FOR_START) && bit)
-	return;
+  if( state == RCSTA_MAYBE_START) {
+    if (bit)
+      state = RCSTA_WAITING_FOR_START;
+    else
+      state = RCSTA_RECEIVING;
+    return;
+  }
+  if (bit_count == 0) {
+    // we should now have the stop bit
+    if (bit) {
+      // got the stop bit
+      // If the rxreg has data from a previous reception then
+      // we have a receiver overrun error.
+      // cout << "rcsta.rsr is full\n";
 
-      // Copy the bit into the Receive Shift Register
-      if(bit)
-	rsr |= 1<<9;
+      if((value.get() & RX9) == 0)
+        rsr >>= 1;
 
-      //cout << "Receive bit #" << bit_count << ": " << (rsr&(1<<9)) << '\n';
+      // copy the rsr to the fifo
+      if(rcreg)
+        rcreg->push( rsr & 0xff);
 
-      rsr >>= 1;
-
-      if(--bit_count == 0)
-	{
-	  // rsr is full.
-
-	  // If the rxreg has data from a previous reception then
-	  // we have a receiver overrun error.
-	  //cout << "rcsta.rsr is full\n";
-
-	  if((value & RX9) == 0)
-	    rsr >>= 1;
-
-	  // copy the rsr to the fifo
-	  rcreg->push( rsr & 0xff);
-
-	  // If we're continuously receiving, then set up for the next byte.
-	  // FIXME -- may want to set a half bit delay before re-starting...
-	  if(value & CREN)
-	    start_receiving();
-	  else
-	    state = RCSTA_DISABLED;
-
-	}
+      Dprintf(("_RCSTA::receive_a_bit received 0x%02X\n",rsr & 0xff));
 
+    } else {
+      //not stop bit; discard the data and go back receiving
     }
+    // If we're continuously receiving, then set up for the next byte.
+    // FIXME -- may want to set a half bit delay before re-starting...
+    if(value.get() & CREN)
+      start_receiving();
+    else
+      state = RCSTA_DISABLED;
+    return;
+  }
+
+
+  // Copy the bit into the Receive Shift Register
+  if(bit)
+    rsr |= 1<<9;
+
+  //cout << "Receive bit #" << bit_count << ": " << (rsr&(1<<9)) << '\n';
+
+  rsr >>= 1;
+  bit_count--;
 
 }
 
-void _RCSTA::stop_receiving(void)
+void _RCSTA::stop_receiving()
 {
 
   rsr = 0;
@@ -420,7 +612,7 @@ void _RCSTA::stop_receiving(void)
 
 }
 
-void _RCSTA::start_receiving(void)
+void _RCSTA::start_receiving()
 {
   //cout << "The USART is starting to receive data\n";
 
@@ -428,89 +620,100 @@ void _RCSTA::start_receiving(void)
   sample = 0;
 
   // Is this a 9-bit data reception?
-  if(value & RX9)
+  if(value.get() & RX9)
     {
-      bit_count = 10;
+      bit_count = 9;
     }
   else
     {
-      bit_count = 9;
+      bit_count = 8;
     }
 
   state = RCSTA_WAITING_FOR_START;
 
 }
+void _RCSTA::overrun()
+{
+  value.put(value.get() | _RCSTA::OERR);
+}
 
 void _RCSTA::set_callback_break(unsigned int spbrg_edge)
 {
-  //  last_cycle = cpu->cycles.value;
+  //  last_cycle = cycles.value;
 
-  if(cpu)
-    cpu->cycles.set_break(cpu->cycles.value + (spbrg->value + 1) * spbrg_edge, this);
+  if(cpu && spbrg)
+    get_cycles().set_break(get_cycles().value + (spbrg->value.get() + 1) * spbrg_edge, this);
 
 }
-void _RCSTA::receive_start_bit(void)
+void _RCSTA::receive_start_bit()
 {
+  Dprintf(("USART received a start bit\n"));
 
-  //cout << "USART received a start bit\n";
-
-  if(txsta->value & _TXSTA::BRGH)
+  if((value.get() & (CREN | SREN)) == 0) {
+    Dprintf(("  but not enabled\n"));
+    return;
+  }
+  
+  if(txsta && (txsta->value.get() & _TXSTA::BRGH))
     set_callback_break(BRGH_FIRST_MID_SAMPLE);
   else
-    set_callback_break(BRGH_FIRST_MID_SAMPLE);
+    set_callback_break(BRGL_FIRST_MID_SAMPLE);
 
   sample = 0;
-  state = RCSTA_WAITING_MID1;
-
+  sample_state = RCSTA_WAITING_MID1;
+  state = RCSTA_MAYBE_START;
 }
 
-void _RCSTA::callback(void)
+//------------------------------------------------------------
+void _RCSTA::callback()
 {
 
-  //cout << "RCSTA callback " << (cpu->cycles.value) << '\n';
+  Dprintf(("RCSTA callback. time:0x%llx\n",cycles.value));
 
-  switch(state) {
+  // A bit is sampled 3 times.
+
+  switch(sample_state) {
   case RCSTA_WAITING_MID1:
-    if(uart_port->get_bit(rx_bit))
+    if (m_cRxState == '1' || m_cRxState == 'W')
       sample++;
 
-    if(txsta->value & _TXSTA::BRGH)
+    if(txsta && (txsta->value.get() & _TXSTA::BRGH))
       set_callback_break(BRGH_SECOND_MID_SAMPLE - BRGH_FIRST_MID_SAMPLE);
     else
       set_callback_break(BRGL_SECOND_MID_SAMPLE - BRGL_FIRST_MID_SAMPLE);
 
-    state = RCSTA_WAITING_MID2;
+    sample_state = RCSTA_WAITING_MID2;
 
     break;
 
   case RCSTA_WAITING_MID2:
-    if(uart_port->get_bit(rx_bit))
+    if (m_cRxState == '1' || m_cRxState == 'W')
       sample++;
 
-    if(txsta->value & _TXSTA::BRGH)
+    if(txsta && (txsta->value.get() & _TXSTA::BRGH))
       set_callback_break(BRGH_THIRD_MID_SAMPLE - BRGH_SECOND_MID_SAMPLE);
     else
       set_callback_break(BRGL_THIRD_MID_SAMPLE - BRGL_SECOND_MID_SAMPLE);
 
-    state = RCSTA_WAITING_MID3;
+    sample_state = RCSTA_WAITING_MID3;
 
     break;
 
   case RCSTA_WAITING_MID3:
-    if(uart_port->get_bit(rx_bit))
+    if (m_cRxState == '1' || m_cRxState == 'W')
       sample++;
 
     receive_a_bit( (sample>=2));
     sample = 0;
 
     // If this wasn't the last bit then go ahead and set a break for the next bit.
-    if(state==RCSTA_WAITING_MID3) {
-      if(txsta->value & _TXSTA::BRGH)
+    if(state==RCSTA_RECEIVING) {
+      if(txsta && (txsta->value.get() & _TXSTA::BRGH))
 	set_callback_break(TOTAL_BRGH_STATES -(BRGH_THIRD_MID_SAMPLE - BRGH_FIRST_MID_SAMPLE));
       else
 	set_callback_break(TOTAL_BRGL_STATES -(BRGL_THIRD_MID_SAMPLE - BRGL_FIRST_MID_SAMPLE));
 
-      state = RCSTA_WAITING_MID1;
+      sample_state = RCSTA_WAITING_MID1;
     }
 
     break;
@@ -523,52 +726,70 @@ void _RCSTA::callback(void)
 
 }
 
+//-----------------------------------------------------------
+void _RCSTA::callback_print()
+{
+  cout << "RCSTA " << name() << " CallBack ID " << CallBackID << '\n';
+}
+
 //-----------------------------------------------------------
 // RCREG
 //
 void _RCREG::push(unsigned int new_value)
 {
-  if(fifo_sp >= 2)
-    {
-      rcsta->value |= _RCSTA::OERR;
-      if(verbose)
-	cout << "receive overrun\n";
-    }
-  else
-    {
-      //cout << "pushing uart reception onto rcreg stack\n";
-      fifo_sp++;
-      oldest_value = value;
-      value = new_value;
-      trace.register_write(address,value);
-    }
+  trace.raw(write_trace.get() | value.get());
+
+  if(fifo_sp >= 2) {
+
+    if (m_rcsta)
+      m_rcsta->overrun();
+      
+    Dprintf(("receive overrun\n"));
+
+  } else {
+
+    Dprintf(("pushing uart reception onto rcreg stack, value received=0x%x\n",new_value));
+    fifo_sp++;
+    oldest_value = value.get();
+    value.put(new_value);
+  }
+
+  if(pir_set)
+    pir_set->set_rcif();
 
 }
 
-void _RCREG::pop(void)
+void _RCREG::pop()
 {
 
   if(fifo_sp == 0)
     return;
 
   if(--fifo_sp == 1)
-    value = oldest_value;
+    value.put(oldest_value);
+
+  if(fifo_sp == 0 && pir_set)
+    pir_set->clear_rcif();
 
 }
 
-unsigned int _RCREG::get_value(void)
+unsigned int _RCREG::get_value()
 {
 
-  return value;
+  return value.get();
 
 }
 
-unsigned int _RCREG::get(void)
+unsigned int _RCREG::get()
 {
 
   pop();
-  trace.register_read(address,value);
-  return value;
+  trace.raw(read_trace.get() | value.get());
+  return value.get();
+}
+void _RCREG::assign_pir_set(PIR_SET *new_pir_set)
+{
+  pir_set = new_pir_set;
 }
 
 
@@ -580,64 +801,59 @@ unsigned int _RCREG::get(void)
 // serial port is not used. Perhaps gpsim needs some kind of
 // pragma type thing to disable cpu intensive peripherals...)
 
-void _SPBRG::get_next_cycle_break(void)
+void _SPBRG::get_next_cycle_break()
 {
 
-  if(txsta->value & _TXSTA::SYNC)
+  if(txsta && (txsta->value.get() & _TXSTA::SYNC))
     {
       // Synchronous mode
-      future_cycle = last_cycle + (value + 1)*4;
+      future_cycle = last_cycle + (value.get() + 1)*4;
     }
   else
     {
       // Asynchronous mode
-      if(txsta->value & _TXSTA::BRGH)
-	future_cycle = last_cycle + (value + 1)*16;
+      if(txsta && (txsta->value.get() & _TXSTA::BRGH))
+	future_cycle = last_cycle + (value.get() + 1)*16;
       else
-	future_cycle = last_cycle + (value + 1)*64;
+	future_cycle = last_cycle + (value.get() + 1)*64;
     }
 
   if(cpu)
-    cpu->cycles.set_break(future_cycle, this);
+    get_cycles().set_break(future_cycle, this);
 
-  //  cout << "SPBRG::callback next break at 0x" << hex << future_cycle <<'\n';
+  Dprintf(("SPBRG::callback next break at 0x%llx\n",future_cycle));
   
 }
 
-void _SPBRG::start(void)
+void _SPBRG::start()
 {
 
-  if(verbose)
-    cout << "SPBRG::start\n";
-
   if(cpu)
-    last_cycle = cpu->cycles.value;
+    last_cycle = get_cycles().value;
   start_cycle = last_cycle;
 
   get_next_cycle_break();
 
-  if(verbose)
-    cout << " SPBRG::start   last_cycle = " << 
-      hex << last_cycle << " future_cycle = " << future_cycle << '\n';
+  Dprintf((" SPBRG::start   last_cycle:0x%llx: future_cycle:0x%llx\n",last_cycle,future_cycle));
 
-  if(verbose && cpu)
-    cpu->cycles.dump_breakpoints();
 
 }
 
 //--------------------------
-//guint64 _SPBRG::get_last_cycle(void)
+//guint64 _SPBRG::get_last_cycle()
 //
 // Get the cpu cycle corresponding to the last edge of the SPBRG
 //
 
-guint64 _SPBRG::get_last_cycle(void)
+guint64 _SPBRG::get_last_cycle()
 {
 
   // There's a chance that a SPBRG break point exists on the current
-  // cpu cycle, but has not yet been serviced. 
-  return( (cpu->cycles.value == future_cycle) ? future_cycle : last_cycle);
-
+  // cpu cycle, but has not yet been serviced.
+  if(cpu)
+    return( (get_cycles().value == future_cycle) ? future_cycle : last_cycle);
+  else
+    return 0;
 }
 //--------------------------
 //guint64 _SPBRG::get_cpu_cycle(unsigned int edges_from_now)
@@ -663,21 +879,19 @@ guint64 _SPBRG::get_cpu_cycle(unsigned int edges_from_now)
 
   // There's a chance that a SPBRG break point exists on the current
   // cpu cycle, but has not yet been serviced. 
-  guint64 cycle = (cpu->cycles.value == future_cycle) ? future_cycle : last_cycle;
+  guint64 cycle = (get_cycles().value == future_cycle) ? future_cycle : last_cycle;
 
-  if(txsta->value & _TXSTA::SYNC)
-    {
-      // Synchronous mode
-      return ( edges_from_now * (value + 1)*4 + cycle);
-    }
-  else
-    {
+  if(txsta && (txsta->value.get() & _TXSTA::SYNC))
+    // Synchronous mode
+    return ( edges_from_now * (value.get() + 1)*4 + cycle);
+
+  else  {
       // Asynchronous mode
-      if(txsta->value & _TXSTA::BRGH)
-	return ( edges_from_now * (value + 1)*16 + cycle);
-      else
-	return ( edges_from_now * (value + 1)*64 + cycle);
-    }
+    if(txsta && (txsta->value.get() & _TXSTA::BRGH))
+      return ( edges_from_now * (value.get() + 1)*16 + cycle);
+    else
+      return ( edges_from_now * (value.get() + 1)*64 + cycle);
+  }
 
 
 #if 0
@@ -689,33 +903,33 @@ const guint64 SPBRG_ASYNC_HI = ~((guint64) 15);
   guint64 cycle;
 
   // Get the number of cycles the sprbg has been active
-  cycle = (cpu->cycles.value - start_cycle);
+  cycle = (cycles.value - start_cycle);
 
-  if(txsta->value & _TXSTA::SYNC)
+  if(txsta->value.get() & _TXSTA::SYNC)
     {
       // Synchronous mode
-      cycle = (cycle & SPBRG_SYNC_MASK) + edges_from_now * (value + 1)*4 + start_cycle;
+      cycle = (cycle & SPBRG_SYNC_MASK) + edges_from_now * (value.get() + 1)*4 + start_cycle;
     }
   else
     {
       // Asynchronous mode
       if(txsta->value & _TXSTA::BRGH)
-	cycle = (cycle & SPBRG_ASYNC_HI) + edges_from_now * (value + 1)*16 + start_cycle;
+	cycle = (cycle & SPBRG_ASYNC_HI) + edges_from_now * (value.get() + 1)*16 + start_cycle;
       else
-	cycle = (cycle & SPBRG_ASYNC_LO) + edges_from_now * (value + 1)*64 + start_cycle;
+	cycle = (cycle & SPBRG_ASYNC_LO) + edges_from_now * (value.get() + 1)*64 + start_cycle;
     }
 
   return cycle;
 #endif
 }
-void _SPBRG::callback(void)
+void _SPBRG::callback()
 {
 
-  last_cycle = cpu->cycles.value;
+  last_cycle = get_cycles().value;
 
-  //cout << "SPBRG rollover at cycle " << last_cycle << '\n';
+  Dprintf(("SPBRG rollover at cycle:0x%llx\n",last_cycle));
 
-  if(rcsta->value & _RCSTA::SPEN)
+  if(rcsta && (rcsta->value.get() & _RCSTA::SPEN))
     {
 
       // If the serial port is enabled, then set another 
@@ -725,20 +939,23 @@ void _SPBRG::callback(void)
     }
 }
 //--------------------------------------------------
-
-bool TXREG_14::is_empty(void)
+/*
+bool TXREG_14::is_empty()
 {
-  return(pir1->get_txif());
+  return(pir_set->get_txif());
 }
 
-void TXREG_14::empty(void)
+void TXREG_14::empty()
 {
-  pir1->set_txif();
+  Dprintf(("txreg::empty - setting TXIF\n"));
+  pir_set->set_txif();
 }
 
-void TXREG_14::full(void)
+void TXREG_14::full()
 {
-  pir1->clear_txif();
+  Dprintf(("txreg::full - clearing TXIF\n"));
+  if(pir_set)
+    pir_set->clear_txif();
 }
 
 void RCREG_14::push(unsigned int new_value)
@@ -746,124 +963,59 @@ void RCREG_14::push(unsigned int new_value)
 
   _RCREG::push(new_value);
 
-  pir1->set_rcif();
+  pir_set->set_rcif();
 
 }
 
-void RCREG_14::pop(void)
+void RCREG_14::pop()
 {
 
   _RCREG::pop();
   if(fifo_sp == 0)
-    pir1->clear_rcif();
+    pir_set->clear_rcif();
 
 }
-
+*/
 
 //--------------------------------------------------
 // member functions for the USART
 //--------------------------------------------------
-void USART_MODULE::initialize(IOPORT *uart_port)
+void USART_MODULE::initialize(PIR_SET *pir_set,
+			      PinModule *tx_pin, PinModule *rx_pin,
+			      _TXREG *_txreg, _RCREG *_rcreg)
 {
+  assert(_txreg && _rcreg);
 
-  if(spbrg) {
-    spbrg->txsta = txsta;
-    spbrg->rcsta = rcsta;
-  }
+  spbrg.txsta = &txsta;
+  spbrg.rcsta = &rcsta;
 
-  if(txreg) {
-    txreg->assign_pir(NULL);
-    txreg->txsta = txsta;
-  }
+  txreg = _txreg;
+  txreg->assign_pir_set(pir_set);
+  txreg->assign_txsta(&txsta);
 
-  if(txsta) {
-    txsta->txreg = txreg;
-    txsta->spbrg = spbrg;
-    txsta->txpin = NULL; //uart_port->pins[6];
-    txsta->bit_count = 0;
-  }
+  rcreg = _rcreg;
+  rcreg->assign_pir_set(pir_set);
+  rcreg->assign_rcsta(&rcsta);
 
-  if(rcsta) {
-    rcsta->rcreg = rcreg;
-    rcsta->spbrg = spbrg;
-    rcsta->txsta = txsta;
-    rcsta->uart_port = uart_port;
-    rcsta->rx_bit = 7;
-  }
+  txsta.txreg = txreg;
+  txsta.spbrg = &spbrg;
+  txsta.bit_count = 0;
+  txsta.setIOpin(tx_pin);
 
-  if(rcreg) {
-    rcreg->assign_pir(NULL);
-    rcreg->rcsta = rcsta;
-  }
+  rcsta.rcreg = rcreg;
+  rcsta.spbrg = &spbrg;
+  rcsta.txsta = &txsta;
+  rcsta.setIOpin(rx_pin);
 
 }
 
 //--------------------------------------------------
-USART_MODULE::USART_MODULE(void)
+USART_MODULE::USART_MODULE()
+  : txreg(0), rcreg(0)
 {
-
-  txreg = NULL;
-  rcreg = NULL;
-
-  rcsta = new _RCSTA;
-  txsta = new _TXSTA;
-
 }
 
 //--------------------------------------------------
-USART_MODULE14::USART_MODULE14(void)
+USART_MODULE14::USART_MODULE14()
 {
-
-  txreg = new TXREG_14;
-  rcreg = new RCREG_14;
-
-  spbrg = new _SPBRG;
-
-}
-
-//--------------------------------------------------
-void USART_MODULE14::initialize(_14bit_processor *new_cpu, PIR1 *pir1, IOPORT *uart_port)
-{
-  cpu = new_cpu;
-
-  USART_MODULE::initialize(uart_port);
-
-  //spbrg.txsta = &txsta;
-  //spbrg.rcsta = &rcsta;
-
-  if(txreg) //this should be unnecessary
-    txreg->assign_pir(pir1);
-
-  //txreg.txsta = &txsta;
-
-  //txsta.txreg = &txreg;
-  //txsta.spbrg = &spbrg;
-  if(txsta)
-    txsta->txpin = uart_port->pins[6];
-  //txsta.bit_count = 0;
-
-  //rcsta.rcreg = &rcreg;
-  //rcsta.spbrg = &spbrg;
-  //rcsta.txsta = &txsta;
-  //rcsta.uart_port = uart_port;
-  //rcsta.rx_bit = 7;
-
-  //rcreg.rcsta = &rcsta;
-  if(rcreg) 
-    rcreg->assign_pir(pir1);
-
-  //  spbrg.start();
-
-}
-
-// This gets called whenever there's a change detected on the RX pin.
-// The usart is only interested in those changes when it is waiting
-// for the start bit. Otherwise, the rcsta callback function will sample
-// the rx pin (if we're receiving).
-
-void   USART_MODULE14::new_rx_edge(unsigned int bit)
-{
-  if(rcsta)
-    if( (rcsta->state == _RCSTA::RCSTA_WAITING_FOR_START) && !bit)
-      rcsta->receive_start_bit();
 }
